import 'dart:async';
import 'package:flutter/material.dart';
import 'package:masamune_flutter/masamune_flutter.dart';
import 'uitextfieldcontrollermixin.dart';

/// Mixins for using forms on pages.
abstract class UIFormMixin {
  /// Key for form.
  final formKey = GlobalKey<FormState>();

  /// Data for the form.
  IDataDocument get form => this._form;
  final _form = DataDocument(DefaultPath.formData);

  /// Validate and save the data in the form.
  ///
  /// Returns True if the validation is successful.
  bool validate() {
    if (this.formKey == null || this.formKey.currentState == null) return false;
    if (!this.formKey.currentState.validate()) return false;
    this._form.clear();
    this.formKey.currentState.save();
    return true;
  }

  Future<IDataDocument> save(FutureOr<IDataDocument> target,
      {List<String> whiteList, List<String> blackList}) async {
    if (this.formKey == null ||
        this.formKey.currentState == null ||
        this.form == null) return target;
    if (target is Future<IDataDocument>) {
      return target.then((value) async {
        for (MapEntry<String, IDataField> tmp in value.entries) {
          if (isEmpty(tmp.key) || tmp.value == null) continue;
          if (whiteList != null &&
              whiteList.length > 0 &&
              !whiteList.contains(tmp.key)) continue;
          if (blackList != null &&
              blackList.length > 0 &&
              blackList.contains(tmp.key)) continue;
          value[tmp.key] = tmp.value.data;
        }
        await value.save();
        return value;
      });
    } else if (target is IDataDocument) {
      for (MapEntry<String, IDataField> tmp in target.entries) {
        if (isEmpty(tmp.key) || tmp.value == null) continue;
        if (whiteList != null &&
            whiteList.length > 0 &&
            !whiteList.contains(tmp.key)) continue;
        if (blackList != null &&
            blackList.length > 0 &&
            blackList.contains(tmp.key)) continue;
        target[tmp.key] = tmp.value.data;
      }
      await target.save();
      return target;
    }
    return null;
  }


}
